use tokio::io::AsyncWrite;

use lavina_core::prelude::Str;
use proto_irc::client::command_args::Whois;
use proto_irc::response::{IrcResponseMessage, WriteResponse};

use crate::commands::whois::error::{ErrNoNicknameGiven431, ErrNoSuchNick401};
use crate::commands::whois::response::{RplEndOfWhois318, RplWhoIsUser311};
use crate::commands::Handler;

use super::HandlerArgs;

pub mod error;
pub mod response;

impl<T: AsyncWrite + Unpin> Handler<T> for Whois {
    async fn handle(&self, body: HandlerArgs<T>) -> anyhow::Result<()> {
        match self {
            Whois::Nick(nick) => handle_nick_target(nick.clone(), body).await?,
            Whois::TargetNick(_, nick) => handle_nick_target(nick.clone(), body).await?,
            Whois::EmptyArgs => {
                let HandlerArgs {
                    server_name,
                    mut writer,
                    ..
                } = body;
                IrcResponseMessage::empty_tags(
                    Some(server_name.clone()),
                    ErrNoNicknameGiven431::new(server_name.clone()),
                )
                .write_response(&mut writer)
                .await?
            }
        }
        Ok(())
    }
}

async fn handle_nick_target(nick: Str, body: HandlerArgs<impl AsyncWrite + Unpin>) -> anyhow::Result<()> {
    let HandlerArgs {
        server_name,
        mut writer,
        client,
        mut storage,
    } = body;
    if let Some(user) = storage.retrieve_user_by_name(nick.clone().as_ref()).await? {
        IrcResponseMessage::empty_tags(
            Some(server_name.clone()),
            RplWhoIsUser311::new(
                client.clone(),
                nick.clone(),
                Some(Str::from(user.name.clone())),
                server_name.clone(),
                nick.clone(),
            ),
        )
        .write_response(&mut writer)
        .await?;

        IrcResponseMessage::empty_tags(
            Some(server_name.clone()),
            RplEndOfWhois318::new(client.clone(), user.name.clone().into()),
        )
        .write_response(&mut writer)
        .await?
    } else {
        ErrNoSuchNick401::new(client.clone(), nick.clone()).write_response(&mut writer).await?
    }
    Ok(())
}
